/* Copyright (C) 2022 INRIA

 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <pari/pari.h>

int main (int argc, char **argv)
{
    int i, n;
    char ifname[5], ofname[3][6];
    FILE *ifile, *ofile[3];
    pari_sp av;
    long disc, c[5];
    GEN gdisc, fmat, r, rabs, nf, bnf;
    long r1, r2;
    GEN coef, no, cyc;

    if (argc == 2)
        n = atoi (argv[1]);
    else
        return 1;

    sprintf (ifname, "i_%02d", n);
    ifile = fopen (ifname, "r");

    for (i = 0; i < 3; i++) {
        sprintf (ofname[i], "o%d_%02d", i, n);
        ofile[i] = fopen (ofname[i], "w");
    }
    pari_init (1000000000, 1000000);

    r = cgetg (7, t_POL);
    r[1] = evalvarn (0) | evalsigne (1);

    av = avma;

    while (fscanf (ifile, " %ld , %ld, %ld, %ld, %ld, %ld",
                   &disc, c, c + 1, c + 2, c + 3, c + 4) != EOF) {

        gdisc = stoi (disc);
        for (i = 0; i < 5; i++)
            gel (r, 6 - i) = stoi (c[i]);

        fmat = matconcat (mkcol2 (factor (gdisc), factor (gel (r, 6))));
        nf = nfinitall (mkvec2 (r, fmat), nf_RED, DEFAULTPREC);

        if (lg (nfcertify (nf)) > 1) {
            pari_printf ("%Ps\n", r);
            continue;
	}

        rabs = polredabs (nf);
        bnf = bnfinit0 (nf, 0, NULL, DEFAULTPREC);

        nf_get_sign (nf, &r1, &r2);
        coef = gtovec0 (rabs, 0);
        no = bnf_get_no (bnf);
        cyc = bnf_get_cyc (bnf);

        pari_fprintf (ofile[r2], "%Ps\n", mkvec4 (gdisc, coef, no, cyc));

        set_avma (av);
    }
    fclose (ifile);
    for (i = 0; i < 3; i++)
        fclose (ofile[i]);

    return 0;
}
